@@ -1,3 +1,5 @@
+# Copyright (c) 2022 Microsoft
+# Licensed under The MIT License [see LICENSE for details]
 
 
 class EncoderConfig(object):
@@ -7,6 +9,7 @@ class EncoderConfig(object):
         self.encoder_ffn_embed_dim = kwargs.pop("encoder_ffn_embed_dim", 3072)
         self.encoder_layers = kwargs.pop("encoder_layers", 12)
         self.encoder_normalize_before = kwargs.pop("encoder_normalize_before", True)
+        self.normalize_output = kwargs.pop("normalize_output", True)
         self.activation_fn = kwargs.pop("activation_fn", "gelu")
         self.dropout = kwargs.pop("dropout", 0.0)
         self.drop_path_rate = kwargs.pop("drop_path_rate", 0.0)
@@ -18,19 +21,26 @@ class EncoderConfig(object):
         self.moe_top1_expert = kwargs.pop("moe_top1_expert", False)
         self.moe_expert_count = kwargs.pop("moe_expert_count", 0)
         self.moe_gating_use_fp32 = kwargs.pop("moe_gating_use_fp32", True)
-        self.moe_eval_capacity_token_fraction = kwargs.pop("moe_eval_capacity_token_fraction", 0.25)
+        self.moe_eval_capacity_token_fraction = kwargs.pop(
+            "moe_eval_capacity_token_fraction", 0.25
+        )
         self.moe_second_expert_policy = kwargs.pop("moe_second_expert_policy", "random")
-        self.moe_normalize_gate_prob_before_dropping = kwargs.pop("moe_normalize_gate_prob_before_dropping", False)
-        self.use_xmoe = kwargs.pop("use_xmoe", True)
+        self.moe_normalize_gate_prob_before_dropping = kwargs.pop(
+            "moe_normalize_gate_prob_before_dropping", False
+        )
+        self.use_xmoe = kwargs.pop("use_xmoe", False)
         self.rel_pos_buckets = kwargs.pop("rel_pos_buckets", 0)
         self.max_rel_pos = kwargs.pop("max_rel_pos", 0)
         self.deepnorm = kwargs.pop("deepnorm", False)
         self.subln = kwargs.pop("subln", True)
         self.bert_init = kwargs.pop("bert_init", False)
         self.multiway = kwargs.pop("multiway", False)
-        self.share_encoder_input_output_embed = kwargs.pop("share_encoder_input_output_embed", False)
+        self.share_encoder_input_output_embed = kwargs.pop(
+            "share_encoder_input_output_embed", False
+        )
         self.max_source_positions = kwargs.pop("max_source_positions", 1024)
         self.no_output_layer = kwargs.pop("no_output_layer", False)
+        self.layernorm_eps = kwargs.pop("layernorm_eps", 1e-5)
         # Text
         self.vocab_size = kwargs.pop("vocab_size", -1)
         # Vision
@@ -41,11 +51,19 @@ class EncoderConfig(object):
         self.checkpoint_activations = kwargs.pop("checkpoint_activations", False)
         self.fsdp = kwargs.pop("fsdp", False)
         self.ddp_rank = kwargs.pop("ddp_rank", 0)
+        self.xpos_rel_pos = kwargs.pop("xpos_rel_pos", False)
+        self.xpos_scale_base = kwargs.pop("xpos_scale_base", 512)
 
         if self.deepnorm:
             self.encoder_normalize_before = False
+            self.subln = False
         if self.subln:
             self.encoder_normalize_before = True
+            self.deepnorm = False
+        if self.use_xmoe:
+            self.moe_normalize_gate_prob_before_dropping = True
+            self.moe_second_expert_policy = "random"
+            assert self.moe_freq > 0 and self.moe_expert_count > 0
 
     def override(self, args):
         for hp in self.__dict__.keys():
@@ -71,30 +89,45 @@ class DecoderConfig(object):
         self.moe_top1_expert = kwargs.pop("moe_top1_expert", False)
         self.moe_expert_count = kwargs.pop("moe_expert_count", 0)
         self.moe_gating_use_fp32 = kwargs.pop("moe_gating_use_fp32", True)
-        self.moe_eval_capacity_token_fraction = kwargs.pop("moe_eval_capacity_token_fraction", 0.25)
+        self.moe_eval_capacity_token_fraction = kwargs.pop(
+            "moe_eval_capacity_token_fraction", 0.25
+        )
         self.moe_second_expert_policy = kwargs.pop("moe_second_expert_policy", "random")
-        self.moe_normalize_gate_prob_before_dropping = kwargs.pop("moe_normalize_gate_prob_before_dropping", False)
-        self.use_xmoe = kwargs.pop("use_xmoe", True)
+        self.moe_normalize_gate_prob_before_dropping = kwargs.pop(
+            "moe_normalize_gate_prob_before_dropping", False
+        )
+        self.use_xmoe = kwargs.pop("use_xmoe", False)
         self.rel_pos_buckets = kwargs.pop("rel_pos_buckets", 0)
         self.max_rel_pos = kwargs.pop("max_rel_pos", 0)
         self.deepnorm = kwargs.pop("deepnorm", False)
         self.subln = kwargs.pop("subln", True)
         self.bert_init = kwargs.pop("bert_init", False)
         self.multiway = kwargs.pop("multiway", False)
-        self.share_decoder_input_output_embed = kwargs.pop("share_decoder_input_output_embed", False)
+        self.share_decoder_input_output_embed = kwargs.pop(
+            "share_decoder_input_output_embed", False
+        )
         self.max_target_positions = kwargs.pop("max_target_positions", 1024)
         self.no_output_layer = kwargs.pop("no_output_layer", False)
+        self.layernorm_eps = kwargs.pop("layernorm_eps", 1e-5)
         # Text
         self.vocab_size = kwargs.pop("vocab_size", -1)
         # Fairscale
         self.checkpoint_activations = kwargs.pop("checkpoint_activations", False)
         self.fsdp = kwargs.pop("fsdp", False)
         self.ddp_rank = kwargs.pop("ddp_rank", 0)
+        self.xpos_rel_pos = kwargs.pop("xpos_rel_pos", False)
+        self.xpos_scale_base = kwargs.pop("xpos_scale_base", 512)
 
         if self.deepnorm:
             self.decoder_normalize_before = False
+            self.subln = False
         if self.subln:
             self.decoder_normalize_before = True
+            self.deepnorm = False
+        if self.use_xmoe:
+            self.moe_normalize_gate_prob_before_dropping = True
+            self.moe_second_expert_policy = "random"
+            assert self.moe_freq > 0 and self.moe_expert_count > 0
 
     def override(self, args):
         for hp in self.__dict__.keys():
@@ -125,10 +158,14 @@ class EncoderDecoderConfig(object):
         self.moe_top1_expert = kwargs.pop("moe_top1_expert", False)
         self.moe_expert_count = kwargs.pop("moe_expert_count", 0)
         self.moe_gating_use_fp32 = kwargs.pop("moe_gating_use_fp32", True)
-        self.moe_eval_capacity_token_fraction = kwargs.pop("moe_eval_capacity_token_fraction", 0.25)
+        self.moe_eval_capacity_token_fraction = kwargs.pop(
+            "moe_eval_capacity_token_fraction", 0.25
+        )
         self.moe_second_expert_policy = kwargs.pop("moe_second_expert_policy", "random")
-        self.moe_normalize_gate_prob_before_dropping = kwargs.pop("moe_normalize_gate_prob_before_dropping", False)
-        self.use_xmoe = kwargs.pop("use_xmoe", True)
+        self.moe_normalize_gate_prob_before_dropping = kwargs.pop(
+            "moe_normalize_gate_prob_before_dropping", False
+        )
+        self.use_xmoe = kwargs.pop("use_xmoe", False)
         self.rel_pos_buckets = kwargs.pop("rel_pos_buckets", 0)
         self.max_rel_pos = kwargs.pop("max_rel_pos", 0)
         self.deepnorm = kwargs.pop("deepnorm", False)
@@ -136,23 +173,34 @@ class EncoderDecoderConfig(object):
         self.bert_init = kwargs.pop("bert_init", False)
         self.multiway = kwargs.pop("multiway", False)
         self.share_all_embeddings = kwargs.pop("share_all_embeddings", False)
-        self.share_decoder_input_output_embed = kwargs.pop("share_decoder_input_output_embed", False)
+        self.share_decoder_input_output_embed = kwargs.pop(
+            "share_decoder_input_output_embed", False
+        )
         self.max_source_positions = kwargs.pop("max_source_positions", 1024)
         self.max_target_positions = kwargs.pop("max_target_positions", 1024)
         self.no_output_layer = kwargs.pop("no_output_layer", False)
+        self.layernorm_eps = kwargs.pop("layernorm_eps", 1e-5)
         # Text
         self.vocab_size = kwargs.pop("vocab_size", -1)
         # Fairscale
         self.checkpoint_activations = kwargs.pop("checkpoint_activations", False)
         self.fsdp = kwargs.pop("fsdp", False)
         self.ddp_rank = kwargs.pop("ddp_rank", 0)
+        self.xpos_rel_pos = kwargs.pop("xpos_rel_pos", False)
+        self.xpos_scale_base = kwargs.pop("xpos_scale_base", 512)
 
         if self.deepnorm:
             self.encoder_normalize_before = False
             self.decoder_normalize_before = False
+            self.subln = False
         if self.subln:
             self.encoder_normalize_before = True
             self.decoder_normalize_before = True
+            self.deepnorm = False
+        if self.use_xmoe:
+            self.moe_normalize_gate_prob_before_dropping = True
+            self.moe_second_expert_policy = "random"
+            assert self.moe_freq > 0 and self.moe_expert_count > 0
 
     def override(self, args):
         for hp in self.__dict__.keys():
