@@ -1,20 +1,26 @@
+# Copyright (c) 2022 Microsoft
+# Licensed under The MIT License [see LICENSE for details]
+
 import math
+
+import numpy as np
 import torch
 import torch.nn as nn
-import numpy as np
 from fairscale.nn import checkpoint_wrapper, wrap
-from apex.normalization import FusedLayerNorm as LayerNorm
+
+from torchscale.architecture.utils import init_bert_params
+from torchscale.component.droppath import DropPath
 from torchscale.component.feedforward_network import FeedForwardNetwork, make_experts
 from torchscale.component.multihead_attention import MultiheadAttention
-from torchscale.component.xmoe.routing import Top1Gate, Top2Gate
-from torchscale.component.xmoe.moe_layer import MOELayer
-from torchscale.component.droppath import DropPath
-from torchscale.architecture.utils import init_bert_params
 from torchscale.component.relative_position_bias import RelativePositionBias
-
+from torchscale.component.xmoe.moe_layer import MOELayer
+from torchscale.component.xmoe.routing import Top1Gate, Top2Gate
+try:
+    from apex.normalization import FusedLayerNorm as LayerNorm
+except ModuleNotFoundError:
+    from torch.nn import LayerNorm
 
 class DecoderLayer(nn.Module):
-
     def __init__(
         self,
         args,
@@ -25,10 +31,12 @@ class DecoderLayer(nn.Module):
         super().__init__()
         self.args = args
         self.embed_dim = args.decoder_embed_dim
-        self.dropout_module = torch.nn.Dropout(args.dropout, inplace=True)
+        self.dropout_module = torch.nn.Dropout(args.dropout)
 
         if args.drop_path_rate > 0:
-            drop_path_prob = np.linspace(0, args.drop_path_rate, args.decoder_layers)[depth]
+            drop_path_prob = np.linspace(0, args.drop_path_rate, args.decoder_layers)[
+                depth
+            ]
             self.drop_path = DropPath(drop_path_prob)
         else:
             self.drop_path = None
@@ -37,14 +45,14 @@ class DecoderLayer(nn.Module):
 
         self.normalize_before = args.decoder_normalize_before
 
-        self.self_attn_layer_norm = LayerNorm(self.embed_dim)
+        self.self_attn_layer_norm = LayerNorm(self.embed_dim, eps=args.layernorm_eps)
 
         if not is_encoder_decoder:
             self.encoder_attn = None
             self.encoder_attn_layer_norm = None
         else:
             self.encoder_attn = self.build_encoder_attention(self.embed_dim, args)
-            self.encoder_attn_layer_norm = LayerNorm(self.embed_dim)
+            self.encoder_attn_layer_norm = LayerNorm(self.embed_dim, eps=args.layernorm_eps)
 
         self.is_moe_layer = is_moe_layer
         self.ffn_dim = args.decoder_ffn_embed_dim
@@ -76,7 +84,7 @@ class DecoderLayer(nn.Module):
             experts = make_experts(args, self.embed_dim, self.ffn_dim)
             self.moe_layer = MOELayer(gate, experts, args)
 
-        self.final_layer_norm = LayerNorm(self.embed_dim)
+        self.final_layer_norm = LayerNorm(self.embed_dim, eps=args.layernorm_eps)
 
         if args.deepnorm:
             if is_encoder_decoder:
@@ -86,11 +94,6 @@ class DecoderLayer(nn.Module):
         else:
             self.alpha = 1.0
 
-        if args.subln:
-            self.ffn_layernorm = LayerNorm(self.ffn_dim)
-        else:
-            self.ffn_layernorm = None
-
     def build_ffn(self, embed_dim, args):
         return FeedForwardNetwork(
             embed_dim,
@@ -98,6 +101,7 @@ class DecoderLayer(nn.Module):
             args.activation_fn,
             args.dropout,
             args.activation_dropout,
+            args.layernorm_eps,
             args.subln,
         )
 
@@ -188,9 +192,7 @@ class DecoderLayer(nn.Module):
             x = self.ffn(x)
             l_aux = None
         else:
-            x = x.transpose(0, 1)
             x, l_aux = self.moe_layer(x)
-            x = x.transpose(0, 1)
 
         if self.drop_path is not None:
             x = self.drop_path(x)
@@ -203,7 +205,6 @@ class DecoderLayer(nn.Module):
 
 
 class Decoder(nn.Module):
-
     def __init__(
         self,
         args,
@@ -216,7 +217,7 @@ class Decoder(nn.Module):
         super().__init__(**kwargs)
         self.args = args
 
-        self.dropout_module = torch.nn.Dropout(args.dropout, inplace=True)
+        self.dropout_module = torch.nn.Dropout(args.dropout)
 
         embed_dim = args.decoder_embed_dim
         self.embed_dim = embed_dim
@@ -225,13 +226,17 @@ class Decoder(nn.Module):
         self.embed_tokens = embed_tokens
         self.embed_positions = embed_positions
 
-        if output_projection is None and not args.no_output_layer and args.vocab_size > 0:
+        if (
+            output_projection is None
+            and not args.no_output_layer
+            and args.vocab_size > 0
+        ):
             self.output_projection = self.build_output_projection(args)
         else:
             self.output_projection = output_projection
 
         if args.layernorm_embedding:
-            self.layernorm_embedding = LayerNorm(embed_dim)
+            self.layernorm_embedding = LayerNorm(embed_dim, eps=args.layernorm_eps)
         else:
             self.layernorm_embedding = None
 
@@ -252,12 +257,10 @@ class Decoder(nn.Module):
         self.num_layers = len(self.layers)
 
         if args.decoder_normalize_before:
-            self.layer_norm = LayerNorm(embed_dim)
+            self.layer_norm = LayerNorm(embed_dim, eps=args.layernorm_eps)
         else:
             self.layer_norm = None
 
-        self.output_projection = output_projection
-
         self.self_attn_relative_position = None
         self.cross_attn_relative_position = None
 
@@ -283,7 +286,12 @@ class Decoder(nn.Module):
             else:
                 init_scale = math.pow(8.0 * args.decoder_layers, 0.25)
             for name, p in self.named_parameters():
-                if 'fc1' in name or 'fc2' in name or 'out_proj' in name or 'v_proj' in name:
+                if (
+                    "fc1" in name
+                    or "fc2" in name
+                    or "out_proj" in name
+                    or "v_proj" in name
+                ):
                     p.data.div_(init_scale)
 
         if args.subln:
@@ -292,9 +300,14 @@ class Decoder(nn.Module):
             else:
                 init_scale = math.sqrt(math.log(args.decoder_layers * 2))
             for name, p in self.named_parameters():
-                if 'encoder_attn' in name:
+                if "encoder_attn" in name:
                     continue
-                if 'fc1' in name or 'fc2' in name or 'out_proj' in name or 'v_proj' in name:
+                if (
+                    "fc1" in name
+                    or "fc2" in name
+                    or "out_proj" in name
+                    or "v_proj" in name
+                ):
                     p.data.mul_(init_scale)
 
     def build_output_projection(
@@ -313,16 +326,12 @@ class Decoder(nn.Module):
                 args.decoder_embed_dim, args.vocab_size, bias=False
             )
             torch.nn.init.normal_(
-                output_projection.weight, mean=0, std=args.decoder_embed_dim ** -0.5
+                output_projection.weight, mean=0, std=args.decoder_embed_dim**-0.5
             )
         return output_projection
 
     def build_decoder_layer(
-        self,
-        args,
-        depth,
-        is_moe_layer=False,
-        is_encoder_decoder=False
+        self, args, depth, is_moe_layer=False, is_encoder_decoder=False
     ):
         layer = DecoderLayer(
             args,
@@ -344,7 +353,9 @@ class Decoder(nn.Module):
     ):
         positions = None
         if self.embed_positions is not None:
-            positions = self.embed_positions(tokens, incremental_state=incremental_state)
+            positions = self.embed_positions(
+                tokens, incremental_state=incremental_state
+            )
 
         if incremental_state is not None:
             tokens = tokens[:, -1:]
@@ -378,29 +389,28 @@ class Decoder(nn.Module):
         **kwargs
     ):
         # embed tokens and positions
-        x, _ = self.forward_embedding(prev_output_tokens, token_embeddings, incremental_state)
-        x = x.transpose(0, 1)
+        x, _ = self.forward_embedding(
+            prev_output_tokens, token_embeddings, incremental_state
+        )
 
-        # relative postion
+        # relative position
         self_attn_rel_pos_bias = None
         slen = prev_output_tokens.size(1)
         if self.self_attn_relative_position is not None:
             self_attn_rel_pos_bias = self.self_attn_relative_position(
-                batch_size=x.size(1),
-                qlen=slen,
-                klen=slen
+                batch_size=x.size(0), qlen=slen, klen=slen
             )
             if incremental_state is not None:
-                self_attn_rel_pos_bias = self_attn_rel_pos_bias[:, -1:, :]
+                self_attn_rel_pos_bias = self_attn_rel_pos_bias[-1:, :, :]
         cross_attn_rel_pos_bias = None
         if self.cross_attn_relative_position is not None:
             cross_attn_rel_pos_bias = self.cross_attn_relative_position(
-                batch_size=x.size(1),
+                batch_size=x.size(0),
                 qlen=slen,
-                klen=encoder_out["encoder_out"].size(0),
+                klen=encoder_out["encoder_out"].size(1),
             )
             if incremental_state is not None:
-                cross_attn_rel_pos_bias = cross_attn_rel_pos_bias[:, -1:, :]
+                cross_attn_rel_pos_bias = cross_attn_rel_pos_bias[-1:, :, :]
 
         # decoder layers
         inner_states = [x]
@@ -413,7 +423,11 @@ class Decoder(nn.Module):
         for idx, layer in enumerate(self.layers):
             if incremental_state is None:
                 self_attn_mask = torch.triu(
-                    torch.zeros([x.size(0), x.size(0)]).float().fill_(float("-inf")).type_as(x), 1
+                    torch.zeros([x.size(1), x.size(1)])
+                    .float()
+                    .fill_(float("-inf"))
+                    .type_as(x),
+                    1,
                 )
             else:
                 self_attn_mask = None
@@ -423,7 +437,9 @@ class Decoder(nn.Module):
             x, layer_attn, _, l_aux_i = layer(
                 x,
                 encoder_out["encoder_out"] if encoder_out is not None else None,
-                encoder_out["encoder_padding_mask"] if encoder_out is not None else None,
+                encoder_out["encoder_padding_mask"]
+                if encoder_out is not None
+                else None,
                 incremental_state[idx] if incremental_state is not None else None,
                 self_attn_mask=self_attn_mask,
                 self_attn_padding_mask=self_attn_padding_mask,
@@ -436,12 +452,14 @@ class Decoder(nn.Module):
         if self.layer_norm is not None:
             x = self.layer_norm(x)
 
-        x = x.transpose(0, 1)
-
         if not features_only:
             x = self.output_layer(x)
 
-        return x, {"inner_states": inner_states, "l_aux": l_aux, "attn": [layer_attn.mean(dim=0)]}
+        return x, {
+            "inner_states": inner_states,
+            "l_aux": l_aux,
+            "attn": None,
+        }
 
     def output_layer(self, features):
         return self.output_projection(features)
