@@ -1,7 +1,13 @@
+# Copyright (c) 2022 Microsoft
+# Licensed under The MIT License [see LICENSE for details]
+
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-from apex.normalization import FusedLayerNorm as LayerNorm
+try:
+    from apex.normalization import FusedLayerNorm as LayerNorm
+except ModuleNotFoundError:
+    from torch.nn import LayerNorm
 
 
 class set_torch_seed(object):
@@ -32,13 +38,19 @@ class set_torch_seed(object):
 
 
 def make_experts(args, embed_dim, expert_ffn_dim):
-    world_size = 1 if not torch.distributed.is_initialized() else torch.distributed.get_world_size()
+    world_size = (
+        1
+        if not torch.distributed.is_initialized()
+        else torch.distributed.get_world_size()
+    )
     expert_list = []
     ddp_rank = args.ddp_rank
     start_seed = torch.randint(1000000, (1,)).item()
     # at least as many experts than gpus
     if args.moe_expert_count >= world_size:
-        assert args.moe_expert_count % world_size == 0, f'{args.moe_expert_count}, {world_size}'
+        assert (
+            args.moe_expert_count % world_size == 0
+        ), f"{args.moe_expert_count}, {world_size}"
         local_moe_expert_count = args.moe_expert_count // world_size
         for i in range(local_moe_expert_count):
             with set_torch_seed(start_seed + ddp_rank * local_moe_expert_count + i):
@@ -49,11 +61,14 @@ def make_experts(args, embed_dim, expert_ffn_dim):
                         args.activation_fn,
                         args.dropout,
                         args.activation_dropout,
-                        args.subln
+                        args.layernorm_eps,
+                        args.subln,
                     )
                 )
     else:
-        assert world_size % args.moe_expert_count == 0, f'{world_size}, {args.moe_expert_count}'
+        assert (
+            world_size % args.moe_expert_count == 0
+        ), f"{world_size}, {args.moe_expert_count}"
 
         with set_torch_seed(start_seed + ddp_rank % args.moe_expert_count):
             expert_list.append(
@@ -63,7 +78,8 @@ def make_experts(args, embed_dim, expert_ffn_dim):
                     args.activation_fn,
                     args.dropout,
                     args.activation_dropout,
-                    args.subln
+                    args.layernorm_eps,
+                    args.subln,
                 )
             )
     experts = nn.ModuleList(expert_list)
@@ -74,13 +90,12 @@ def get_activation_fn(activation):
     if activation == "relu":
         return F.relu
     elif activation == "gelu":
-        return lambda x: F.gelu(x.float()).type_as(x)
+        return F.gelu
     else:
         raise NotImplementedError
 
 
 class FeedForwardNetwork(nn.Module):
-
     def __init__(
         self,
         embed_dim,
@@ -88,16 +103,17 @@ class FeedForwardNetwork(nn.Module):
         activation_fn,
         dropout,
         activation_dropout,
-        subln=False
+        layernorm_eps,
+        subln=False,
     ):
         super().__init__()
         self.embed_dim = embed_dim
         self.activation_fn = get_activation_fn(activation=str(activation_fn))
-        self.activation_dropout_module = torch.nn.Dropout(activation_dropout, inplace=True)
-        self.dropout_module = torch.nn.Dropout(dropout, inplace=True)
+        self.activation_dropout_module = torch.nn.Dropout(activation_dropout)
+        self.dropout_module = torch.nn.Dropout(dropout)
         self.fc1 = nn.Linear(self.embed_dim, ffn_dim)
         self.fc2 = nn.Linear(ffn_dim, self.embed_dim)
-        self.ffn_layernorm = LayerNorm(ffn_dim) if subln else None
+        self.ffn_layernorm = LayerNorm(ffn_dim, eps=layernorm_eps) if subln else None
 
     def reset_parameters(self):
         self.fc1.reset_parameters()
@@ -109,7 +125,7 @@ class FeedForwardNetwork(nn.Module):
         x_shape = x.shape
         x = x.reshape(-1, x.size(-1))
         x = self.fc1(x)
-        x = self.activation_fn(x)
+        x = self.activation_fn(x.float()).type_as(x)
         x = self.activation_dropout_module(x)
         if self.ffn_layernorm is not None:
             x = self.ffn_layernorm(x)
